// SmultronKit, www.peterborgapps.com/smultron
// Copyright © 2004-2015 Peter Borg Apps AB


#if TARGET_OS_IPHONE
@interface SMLSyntax : NSObject {
}
#else
@interface SMLSyntax : NSObject <NSTextStorageDelegate, NSTextViewDelegate> {
}
#endif

@property BOOL reactToChanges;

@property NSRange lastLineHighlightRange;

@property NSInteger lastCursorPosition;

@property (copy, nonatomic) NSString *functionDefinition;
@property (copy, nonatomic) NSString *removeFromFunction;
@property (copy, nonatomic) NSString *firstString;
@property (copy, nonatomic) NSString *secondString;
@property (copy, nonatomic) NSString *beginCommand;
@property (copy, nonatomic) NSString *endCommand;
@property (copy, nonatomic) NSString *beginInstruction;
@property (copy, nonatomic) NSString *endInstruction;
@property (copy, nonatomic) NSString *firstSingleLineComment;
@property (copy, nonatomic) NSString *secondSingleLineComment;
@property (copy, nonatomic) NSString *beginFirstMultiLineComment;
@property (copy, nonatomic) NSString *endFirstMultiLineComment;
@property (copy, nonatomic) NSString *beginSecondMultiLineComment;
@property (copy, nonatomic) NSString *endSecondMultiLineComment;


#if TARGET_OS_IPHONE
@property (assign, nonatomic) UITextView *firstTextView;
@property (assign, nonatomic) UITextView *secondTextView;
#else
@property (assign, nonatomic) NSTextView *firstTextView;
@property (assign, nonatomic) NSTextView *secondTextView;
#endif

@property (weak, nonatomic) NSLayoutManager *firstLayoutManager;
@property (weak, nonatomic) NSLayoutManager *secondLayoutManager;

@property (copy, nonatomic) NSArray *autocompleteWords;

@property (strong, nonatomic) NSTimer *autocompleteWordsTimer;
@property (strong, nonatomic) NSRegularExpression *firstStringExpression;
@property (strong, nonatomic) NSRegularExpression *secondStringExpression;
@property (copy, nonatomic) NSCharacterSet *letterCharacterSet;
@property (copy, nonatomic) NSCharacterSet *keywordStartCharacterSet;
@property (copy, nonatomic) NSCharacterSet *keywordEndCharacterSet;
@property (copy, nonatomic) NSCharacterSet *attributesCharacterSet;
@property (copy, nonatomic) NSCharacterSet *beginVariable;
@property (copy, nonatomic) NSCharacterSet *endVariable;
@property (copy, nonatomic) NSDictionary *commandsColor;
@property (copy, nonatomic) NSDictionary *commentsColor;
@property (copy, nonatomic) NSDictionary *instructionsColor;
@property (copy, nonatomic) NSDictionary *keywordsColor;
@property (copy, nonatomic) NSDictionary *stringsColor;
@property (copy, nonatomic) NSDictionary *variablesColor;
@property (copy, nonatomic) NSDictionary *attributesColor;
@property (copy, nonatomic) NSDictionary *lineHighlightColor;
@property (copy, nonatomic) NSSet *keywords;

@property (strong, nonatomic) NSMutableIndexSet *commandsIndexSet;
@property (strong, nonatomic) NSMutableIndexSet *instructionsIndexSet;
@property (strong, nonatomic) NSMutableIndexSet *commentsIndexSet;
@property (strong, nonatomic) NSMutableIndexSet *stringsIndexSet;

@property (strong, nonatomic) NSMutableIndexSet *alreadyColoredIndexSet;

@property (copy, nonatomic) NSString *searchString;
@property (copy, nonatomic) NSString *completeString;
@property (copy, nonatomic) NSScanner *scanner;
@property (copy, nonatomic) NSScanner *completeDocumentScanner;

@property (strong, nonatomic) NSMutableArray *allColorsInRangesToSet;

@property NSRange range;
@property NSUInteger temporaryRangeLocation;
@property NSUInteger completeStringLength;
@property NSUInteger searchStringLength;


- (instancetype)initWithTextViews:(NSArray *)textViews NS_DESIGNATED_INITIALIZER;


- (void)recolorPageWithChangedText:(BOOL)changedText;


- (void)prepareKeywordsCharacterSets;
- (void)prepareAttributesCharacterSet;
- (void)prepareLetterCharacterSet;
- (void)prepareFirstString;
- (void)prepareSecondString;

- (void)colorCommands;
- (void)colorInstructions;
- (void)colorKeywords;
- (void)colorVariables;
- (void)colorSecondStringFirstPass;
- (void)colorFirstString;
- (void)colorAttributes;
- (void)colorFirstSingleLineComment;
- (void)colorSecondSingleLineComment;
- (void)colorFirstMultiLineComment;
- (void)colorSecondMultiLineComment;
- (void)colorSecondStringSecondPass;

- (void)addColor:(NSDictionary *)colorDictionary range:(NSRange)range;

#if TARGET_OS_IPHONE
- (void)prepareColorsForPrinting;
#else
- (void)recolorRange:(NSRange)range;
#endif



@end


@protocol SMLSyntaxSubclassProtocol
- (void)prepareSyntax;
- (void)performColoring;
@end
